#layout.minimal-layout div.content form.gipfl-form {
  ul.extensible-set {
    max-width: unset;
    border: 1px solid @gray-light;
  }

  dd.active ul.extensible-set {
    border: 1px solid @icinga-blue;

    input[type=submit]:first-of-type {
      border-width: 1px;
    }
  }

  dd input.related-action[type='submit'] {
    display: none;
  }

  dd.active li.active input.related-action[type='submit'] {
    display: inline-block;
  }

  dd.active ul.extensible-set, ul.extensible-set.sortable {
    input[type=text], select {
      width: 100%;
    }

    input[type=text] {
      // background-color: white;
      border: 1px solid white;
    }
  }
}
