<?php return array(
    'root' => array(
        'name' => 'icinga/incubator',
        'pretty_version' => 'dev-master',
        'version' => 'dev-master',
        'reference' => 'f0625c7ea175b6f56ae764f87ed5be385d47a9a4',
        'type' => 'library',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'dev' => true,
    ),
    'versions' => array(
        'brick/math' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '0.9.3',
            ),
        ),
        'clue/block-react' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v1.5.0',
            ),
        ),
        'clue/connection-manager-extra' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v1.3.0',
            ),
        ),
        'clue/http-proxy-react' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v1.8.0',
            ),
        ),
        'clue/mq-react' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v1.6.0',
            ),
        ),
        'clue/redis-protocol' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v0.3.1',
            ),
        ),
        'clue/redis-react' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v2.6.0',
            ),
        ),
        'clue/soap-react' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v2.0.0',
            ),
        ),
        'clue/socket-raw' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v1.6.0',
            ),
        ),
        'clue/socks-react' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v1.4.0',
            ),
        ),
        'clue/stdio-react' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v2.6.0',
            ),
        ),
        'clue/term-react' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v1.3.0',
            ),
        ),
        'clue/utf8-react' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v1.2.0',
            ),
        ),
        'components/jquery' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '3.6.0',
            ),
        ),
        'cweagans/composer-patches' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '1.7.3',
            ),
        ),
        'doctrine/collections' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '1.8.0',
            ),
        ),
        'doctrine/deprecations' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '1.1.2',
            ),
        ),
        'dompdf/dompdf' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v2.0.3',
            ),
        ),
        'dragonmantank/cron-expression' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v3.3.3',
            ),
        ),
        'erusev/parsedown' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '1.7.4',
            ),
        ),
        'evenement/evenement' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v3.0.2',
            ),
        ),
        'ezyang/htmlpurifier' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v4.17.0',
            ),
        ),
        'fig/http-message-util' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '1.1.5',
            ),
        ),
        'fortawesome/font-awesome' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '6.4.2',
            ),
        ),
        'gipfl/calendar' => array(
            'pretty_version' => 'v0.3.1',
            'version' => '0.3.1.0',
            'reference' => 'fd57d12aa97dd4c2a115e5cae36097ffaed220b8',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/calendar',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/cli' => array(
            'pretty_version' => 'v0.7.0',
            'version' => '0.7.0.0',
            'reference' => 'ea81eece60ee9625177d77f1a9d64d9d33fc22f5',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/cli',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/curl' => array(
            'pretty_version' => 'v0.5.0',
            'version' => '0.5.0.0',
            'reference' => '7f527c0caf4b4a1908be977ae06026701398994a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/curl',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/data-type' => array(
            'pretty_version' => 'v0.4.0',
            'version' => '0.4.0.0',
            'reference' => '931549823e3f2c3b8eea76c6c48a6804b0e3da1e',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/data-type',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/db-migration' => array(
            'pretty_version' => 'v0.1.1',
            'version' => '0.1.1.0',
            'reference' => '12be94b0c83a5e018c9d3d42fe8e98b4cbe248ee',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/db-migration',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/diff' => array(
            'pretty_version' => 'v0.3.0',
            'version' => '0.3.0.0',
            'reference' => 'f9decb9cf20735d6cbe6f508aafd8a720922804b',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/diff',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/format' => array(
            'pretty_version' => 'v0.5.0',
            'version' => '0.5.0.0',
            'reference' => 'c521510424890c762636ad30f5bfcce1dd3ed220',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/format',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/icinga-bundles' => array(
            'pretty_version' => 'v0.8.0',
            'version' => '0.8.0.0',
            'reference' => '7f293e4603df2851e35e4d0ef0c13ff8b4277533',
            'type' => 'metapackage',
            'install_path' => null,
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/icinga-cli-daemon' => array(
            'pretty_version' => 'v0.4.0',
            'version' => '0.4.0.0',
            'reference' => '1372cc123d00ad2ea0ee6cbe5cdc06d8bdf0bfb7',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/icinga-cli-daemon',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/icingaweb2' => array(
            'pretty_version' => 'v0.11.0',
            'version' => '0.11.0.0',
            'reference' => '9bd92f3a4e7a7fa7b00269a76f083651cc3c42c0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/icingaweb2',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/influxdb' => array(
            'pretty_version' => 'v0.5.0',
            'version' => '0.5.0.0',
            'reference' => '0b7b46ceecb1a6c9fcd738973f773d23d97b374a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/influxdb',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/json' => array(
            'pretty_version' => 'v0.3.0',
            'version' => '0.3.0.0',
            'reference' => '1517266a77b5ded2e294fac3b221ee33694ac836',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/json',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/linux-health' => array(
            'pretty_version' => 'v0.2.0',
            'version' => '0.2.0.0',
            'reference' => '4d212535565f353f88c9eef3ecb54e20c40b8b37',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/linux-health',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/log' => array(
            'pretty_version' => 'v0.11.0',
            'version' => '0.11.0.0',
            'reference' => '990f02970f4c0a1f9261ffa19c0f74c377bf3c76',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/log',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/openrpc' => array(
            'pretty_version' => 'v0.2.1',
            'version' => '0.2.1.0',
            'reference' => 'a29b8da2cdaf7d757d86aa0fe6d719cf68fb95ad',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/openrpc',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/process' => array(
            'pretty_version' => 'v0.5.0',
            'version' => '0.5.0.0',
            'reference' => 'b6aa279e4cfd39d42f636f0752384cad0d6d4810',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/process',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/protocol' => array(
            'pretty_version' => 'v0.4.0',
            'version' => '0.4.0.0',
            'reference' => 'fc5983b79d3afd75eff67e675627249cdb836261',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/protocol',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/protocol-jsonrpc' => array(
            'pretty_version' => 'v0.12.0',
            'version' => '0.12.0.0',
            'reference' => 'bf59fd10ac8614e26c383fa2c4458ee20b265392',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/protocol-jsonrpc',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/protocol-netstring' => array(
            'pretty_version' => 'v0.1.1',
            'version' => '0.1.1.0',
            'reference' => 'c0021ea9d69883760ac176121041a05c9a51ca11',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/protocol-netstring',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/react-utils' => array(
            'pretty_version' => 'v0.4.0',
            'version' => '0.4.0.0',
            'reference' => '7a41bc07ac8d28d0331cf069d6e700314f09fdf5',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/react-utils',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/simple-daemon' => array(
            'pretty_version' => 'v0.6.0',
            'version' => '0.6.0.0',
            'reference' => 'b7cd3be54c171d20c6692dbb331a3ae6fe90f276',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/simple-daemon',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/socket' => array(
            'pretty_version' => 'v0.5.0',
            'version' => '0.5.0.0',
            'reference' => 'b26ded48f72d8e72b0efd4a64c834519ff79c5c3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/socket',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/stream' => array(
            'pretty_version' => 'v0.2.0',
            'version' => '0.2.0.0',
            'reference' => 'ae0f8f4fd47a6dcc57885b232e239b0ba82889a3',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/stream',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/systemd' => array(
            'pretty_version' => 'v0.4.0',
            'version' => '0.4.0.0',
            'reference' => 'b61bfe7b6dc3873a274621501ddd35d93a21fb36',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/systemd',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/translation' => array(
            'pretty_version' => 'v0.1.1',
            'version' => '0.1.1.0',
            'reference' => 'e974e6a2f0b524678b6554ee0046b5dd5f70a25d',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/translation',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/web' => array(
            'pretty_version' => 'v0.14.0',
            'version' => '0.14.0.0',
            'reference' => '0cde0d1c8682d04837af334529716bacec9bf528',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/web',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/zfdb' => array(
            'pretty_version' => 'v0.5.0',
            'version' => '0.5.0.0',
            'reference' => 'eb0d0beae79eff4bd75528d2333537d4317bcc21',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/zfdb',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'gipfl/zfdbstore' => array(
            'pretty_version' => 'v0.4.0',
            'version' => '0.4.0.0',
            'reference' => '5a2b9dc0cf7481f2e1b09aed9936db9358641fd0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../gipfl/zfdbstore',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'guzzlehttp/guzzle' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '7.8.0',
            ),
        ),
        'guzzlehttp/promises' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '2.0.1',
            ),
        ),
        'guzzlehttp/psr7' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '2.6.1',
            ),
        ),
        'icinga/incubator' => array(
            'pretty_version' => 'dev-master',
            'version' => 'dev-master',
            'reference' => 'f0625c7ea175b6f56ae764f87ed5be385d47a9a4',
            'type' => 'library',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'ipl/html' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v0.8.0',
            ),
        ),
        'ipl/i18n' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v0.2.1',
            ),
        ),
        'ipl/orm' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v0.6.0',
            ),
        ),
        'ipl/scheduler' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v0.1.2',
            ),
        ),
        'ipl/sql' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v0.7.0',
            ),
        ),
        'ipl/stdlib' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v0.13.0',
            ),
        ),
        'ipl/validator' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v0.5.0',
            ),
        ),
        'ipl/web' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v0.9.1',
            ),
        ),
        'jfcherng/php-color-output' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '2.0.2',
            ),
        ),
        'jfcherng/php-diff' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '6.10.14',
            ),
        ),
        'jfcherng/php-mb-string' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '1.4.8',
            ),
        ),
        'jfcherng/php-sequence-matcher' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '3.2.10',
            ),
        ),
        'masterminds/html5' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '2.8.1',
            ),
        ),
        'phenx/php-font-lib' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '0.5.4',
            ),
        ),
        'phenx/php-svg-lib' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '0.5.0',
            ),
        ),
        'predis/predis' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v2.2.2',
            ),
        ),
        'psr/http-client' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '1.0.3',
            ),
        ),
        'psr/http-factory' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '1.0.2',
            ),
        ),
        'psr/http-message' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '1.1',
            ),
        ),
        'psr/log' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '1.1.4',
            ),
        ),
        'ralouphie/getallheaders' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '3.0.3',
            ),
        ),
        'ramsey/collection' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '1.1.4',
            ),
        ),
        'ramsey/uuid' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '4.2.3',
            ),
        ),
        'react/cache' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v1.2.0',
            ),
        ),
        'react/child-process' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v0.6.5',
            ),
        ),
        'react/datagram' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v1.9.0',
            ),
        ),
        'react/dns' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v1.11.0',
            ),
        ),
        'react/event-loop' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v1.5.0',
            ),
        ),
        'react/http' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v1.9.0',
            ),
        ),
        'react/promise' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v2.11.0',
            ),
        ),
        'react/promise-stream' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v1.6.0',
            ),
        ),
        'react/promise-timer' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v1.10.0',
            ),
        ),
        'react/socket' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v1.14.0',
            ),
        ),
        'react/stream' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v1.3.0',
            ),
        ),
        'ringcentral/psr7' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '1.3.0',
            ),
        ),
        'sabberworm/php-css-parser' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '8.4.0',
            ),
        ),
        'shardj/zf1-future' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '1.23.5',
            ),
        ),
        'simshaun/recurr' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v5.0.2',
            ),
        ),
        'symfony/deprecation-contracts' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v2.5.2',
            ),
        ),
        'symfony/polyfill-ctype' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v1.28.0',
            ),
        ),
        'symfony/polyfill-mbstring' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v1.28.0',
            ),
        ),
        'symfony/polyfill-php80' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v1.28.0',
            ),
        ),
        'symfony/polyfill-php81' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v1.28.0',
            ),
        ),
        'tedivm/jshrink' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v1.7.0',
            ),
        ),
        'webmozart/assert' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '1.11.0',
            ),
        ),
        'wikimedia/less.php' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => 'v3.2.1',
            ),
        ),
    ),
);
